/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.sinfo.flood_duration;

import java.util.ArrayList;
import java.util.Collection;

import org.dive4elements.river.artifacts.common.AbstractCalculationExportableResult;
import org.dive4elements.river.artifacts.common.ExportContextCSV;
import org.dive4elements.river.artifacts.common.ExportContextPDF;
import org.dive4elements.river.artifacts.common.GeneralResultType;
import org.dive4elements.river.artifacts.common.IExportContext;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.common.SInfoResultType;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;

/**
 * Contains the result of a {@link FloodDurationCalculation}.
 *
 * @author Gernot Belger
 */
final class FloodDurationCalculationResult extends AbstractCalculationExportableResult {

    private static final long serialVersionUID = 1L;

    private static final String JASPER_FILE = "/jasper/templates/sinfo.floodduration.jrxml";

    public FloodDurationCalculationResult(final String label, final Collection<ResultRow> rows) {
        super(label, rows);
    }

    @Override
    protected void writeCSVResultMetadata(final ExportContextCSV exportContextCSV) {
        // TODO Metadaten der Wasserspiegellage(n) falls gewählt
        // exportContextCSV.writeCSVWaterlevelMetadata(this.wstInfo);
        // exportContextCSV.writeBlankLine();
        // writer.writeNext(new String[] { "" }); // break line
    }

    @Override
    protected String getJasperFile() {
        // TODO Variante mit Wasserspiegellage(n)
        return JASPER_FILE;
    }

    protected String[] formatRow(final IExportContext exportContextCSV, final ResultRow row) {

        final Collection<String> lines = new ArrayList<>(10);

        lines.add(exportContextCSV.formatRowValue(row, GeneralResultType.station));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.riverside));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.floodDuration));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.floodDischarge));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.infrastructureHeight));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.infrastructuretype));

        // TODO Wasserspiegellage(n) und Dauerzahlen falls gewählt

        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.gaugeLabel));
        lines.add(exportContextCSV.formatRowValue(row, SInfoResultType.location));

        return lines.toArray(new String[lines.size()]);
    }

    @Override
    public void writeCSVHeader(final ExportContextCSV exportContextCSV, final RiverInfo river) {

        final Collection<String> header = new ArrayList<>(20);

        header.add(exportContextCSV.formatCsvHeader(GeneralResultType.station));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.riverside));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.floodDuration));
        header.add(exportContextCSV.msgUnitCSV(SInfoResultType.floodDischarge, SInfoResultType.floodDischarge.getUnit()));
        header.add(exportContextCSV.msgUnitCSV(SInfoResultType.infrastructureHeight, SInfoResultType.infrastructureHeight.getUnit()));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.infrastructuretype));

        // TODO Je vier Spalten der bis zu drei Wasserspiegellagen

        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.gaugeLabel));
        header.add(exportContextCSV.formatCsvHeader(SInfoResultType.location));

        exportContextCSV.writeCSVLine(header.toArray(new String[header.size()]));

    }

    @Override
    protected String[] formatCSVRow(final ExportContextCSV exportContextCSV, final ResultRow row) {

        return this.formatRow(exportContextCSV, row);
    }

    @Override
    protected String[] formatPDFRow(final ExportContextPDF exportContextPDF, final ResultRow row) {

        return this.formatRow(exportContextPDF, row);
    }

    @Override
    protected void addJRTableHeader(final ExportContextPDF exportContextPDF, final MetaAndTableJRDataSource source) {

        /* column headings */
        exportContextPDF.addJRMetadata(source, "station_header", GeneralResultType.station);
        exportContextPDF.addJRMetadata(source, "riverside_header", SInfoResultType.riverside);
        exportContextPDF.addJRMetadata(source, "inundationduration_header", SInfoResultType.floodDuration);
        exportContextPDF.addJRMetadata(source, "inundationduration_q_header", SInfoResultType.floodDischarge);
        exportContextPDF.addJRMetadata(source, "infrastructure_height_header", SInfoResultType.infrastructureHeightFloodDur);
        exportContextPDF.addJRMetadata(source, "infrastructure_type_header", SInfoResultType.infrastructuretype);

        // TODO Je vier Spalten der bis zu drei Wasserspiegellagen

        exportContextPDF.addJRMetadata(source, "gauge_header", SInfoResultType.gaugeLabel);
        exportContextPDF.addJRMetadata(source, "location_header", SInfoResultType.location);
    }
}